Files
TheRock/debug-tools/rocgdb/source/gdb/testsuite/gdb.rocm/multi-inferior-gpu.cpp
T
derek.holloway 7502bf64a4
CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
Flatten subprojects into a single base monorepo
2026-09-02 16:34:47 -07:00

108 lines
2.5 KiB
C++

/* This testcase is part of GDB, the GNU debugger.
Copyright 2023-2024 Free Software Foundation, Inc.
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>. */
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <hip/hip_runtime.h>
#include <stdio.h>
#include <stdlib.h>
#include "rocm-test-utils.h"
__global__ void
kern ()
{
asm ("s_sleep 1");
}
/* Spawn one child process per detected GPU. */
static int __attribute__ ((optnone))
parent (int argc, char **argv)
{
/* Identify how many GPUs we have, and spawn one child for each. */
int num_devices;
CHECK (hipGetDeviceCount (&num_devices));
/* Break here. */
for (int i = 0; i < num_devices; i++)
{
char n[32] = {};
snprintf (n, sizeof (n), "%d", i);
pid_t pid = fork ();
if (pid == -1)
{
perror ("Fork failed");
return -1;
}
if (pid == 0)
{
/* Exec to force the child to re-initialize the ROCm runtime. */
if (execl (argv[0], argv[0], n, nullptr) == -1)
{
perror ("Failed to exec");
return -1;
}
}
}
/* Wait for all children. */
while (true)
{
int ws;
pid_t ret = waitpid (-1, &ws, 0);
if (ret == -1 && errno == ECHILD)
break;
}
/* Last break here. */
int ret = 0;
return ret;
}
static int
child (int argc, char **argv)
{
int dev_number;
if (sscanf (argv[1], "%d", &dev_number) != 1)
{
fprintf (stderr, "Invalid argument \"%s\"\n", argv[1]);
return -1;
}
CHECK (hipSetDevice (dev_number));
kern<<<1, 1>>> ();
CHECK (hipDeviceSynchronize ());
return 0;
}
/* When called with no argument, identify how many AMDGPU devices are
available on the system and spawn one worker process per GPU. If a
command-line argument is provided, it is the index of the GPU to use. */
int
main (int argc, char **argv)
{
if (argc <= 1)
return parent (argc, argv);
else
return child (argc, argv);
}